@@ -57,13 +57,10 @@ struct _EBookBackendMAPIPrivate
 	GThread *update_cache_thread;
 	GCancellable *update_cache;
 
-	EBookBackendSummary *summary;
-	EBookBackendCache *cache;
+	EBookBackendSqliteDB *db;
 	GHashTable *running_book_views;
 };
 
-#define SUMMARY_FLUSH_TIMEOUT_SECS 60
-
 #define ELEMENT_TYPE_MASK   0xF /* mask where the real type of the element is stored */
 
 #define ELEMENT_TYPE_SIMPLE 0x01
@@ -128,46 +125,54 @@ static const struct field_element_mapping {
 static gboolean
 ebbm_get_cache_time (EBookBackendMAPI *ebma, glong *cache_seconds)
 {
+	GError *error = NULL;
 	GTimeVal tv = { 0 };
 	gchar *last_update;
+	gboolean ret = TRUE;
 
 	g_return_val_if_fail (ebma != NULL, FALSE);
 	g_return_val_if_fail (ebma->priv != NULL, FALSE);
-	g_return_val_if_fail (ebma->priv->cache != NULL, FALSE);
+	g_return_val_if_fail (ebma->priv->db != NULL, FALSE);
 	g_return_val_if_fail (cache_seconds != NULL, FALSE);
 
-	last_update = e_book_backend_cache_get_time (ebma->priv->cache);
-	if (!last_update || !g_time_val_from_iso8601 (last_update, &tv)) {
-		g_free (last_update);
-		return FALSE;
-	}
-
+	last_update = e_book_backend_sqlitedb_get_sync_data (ebma->priv->db, EMA_EBB_CACHE_FOLDERID, &error);
+	ret = !error && last_update && g_time_val_from_iso8601 (last_update, &tv);
+	if (error)
+		g_error_free (error);
 	g_free (last_update);
 
-	*cache_seconds = tv.tv_sec;
+	if (ret)
+		*cache_seconds = tv.tv_sec;
 
-	return TRUE;
+	return ret;
 }
 
 static void
 ebbm_set_cache_time (EBookBackendMAPI *ebma, glong cache_seconds)
 {
+	GError *error = NULL;
+	gchar *iso_time = NULL;
+
 	g_return_if_fail (ebma != NULL);
 	g_return_if_fail (ebma->priv != NULL);
-	g_return_if_fail (ebma->priv->cache != NULL);
+	g_return_if_fail (ebma->priv->db != NULL);
 
 	if (cache_seconds > 0) {
-		gchar *iso_time;
 		GTimeVal tv = { 0 };
 
 		tv.tv_sec = cache_seconds;
 		iso_time = g_time_val_to_iso8601 (&tv);
+	}
 
-		e_book_backend_cache_set_time (ebma->priv->cache, iso_time ? iso_time : "0");
+	e_book_backend_sqlitedb_set_sync_data (ebma->priv->db,
+					       EMA_EBB_CACHE_FOLDERID,
+					       iso_time ? iso_time : "0",
+					       &error);
+	g_free (iso_time);
 
-		g_free (iso_time);
-	} else {
-		e_book_backend_cache_set_time (ebma->priv->cache, "0");
+	if (error) {
+		exchange_mapi_debug_print ("%s: Failed to set value: %s", G_STRFUNC, error->message);
+		g_error_free (error);
 	}
 }
 
@@ -242,9 +247,7 @@ ebbm_fetch_contacts (EBookBackendMAPI *ebma, struct mapi_SRestriction *restricti
 	g_return_if_fail (ebmac != NULL);
 	g_return_if_fail (ebmac->op_fetch_contacts != NULL);
 
-	e_file_cache_freeze_changes (E_FILE_CACHE (ebma->priv->cache));
 	ebmac->op_fetch_contacts (ebma, restriction, book_view, &notify_data, error);
-	e_file_cache_thaw_changes (E_FILE_CACHE (ebma->priv->cache));
 
 	if (last_modification_secs && *last_modification_secs < notify_data.last_modification)
 		*last_modification_secs = notify_data.last_modification;
@@ -265,7 +268,7 @@ ebbm_build_cache_update_restriction (EBookBackendMAPI *ebma, TALLOC_CTX *mem_ctx
 
 	priv = ebma->priv;
 	g_return_val_if_fail (priv != NULL, NULL);
-	g_return_val_if_fail (priv->cache != NULL, NULL);
+	g_return_val_if_fail (priv->db != NULL, NULL);
 
 	if (!ebbm_get_cache_time (ebma, &last_update_secs) || last_update_secs <= 0)
 		return NULL;
@@ -288,6 +291,18 @@ ebbm_build_cache_update_restriction (EBookBackendMAPI *ebma, TALLOC_CTX *mem_ctx
 	return restriction;
 }
 
+static gboolean
+unref_backend_idle_cb (gpointer data)
+{
+	EBookBackendMAPI *ebma = data;
+
+	g_return_val_if_fail (ebma != NULL, FALSE);
+
+	g_object_unref (ebma);
+
+	return FALSE;
+}
+
 static gpointer
 ebbm_update_cache_cb (gpointer data)
 {
@@ -302,7 +317,7 @@ ebbm_update_cache_cb (gpointer data)
 
 	priv = ebma->priv;
 	g_return_val_if_fail (priv != NULL, NULL);
-	g_return_val_if_fail (priv->cache != NULL, NULL);
+	g_return_val_if_fail (priv->db != NULL, NULL);
 	g_return_val_if_fail (priv->conn != NULL, NULL);
 
 	ebmac = E_BOOK_BACKEND_MAPI_GET_CLASS (ebma);
@@ -322,10 +337,7 @@ ebbm_update_cache_cb (gpointer data)
 			last_modification_secs = 0;
 
 		ebbm_fetch_contacts (ebma, restriction, NULL, &last_modification_secs, &error);
-		if (!error)
-			e_book_backend_cache_set_populated (priv->cache);
-		else
-			e_file_cache_remove_object (E_FILE_CACHE (priv->cache), "populated");
+		e_book_backend_sqlitedb_set_is_populated (priv->db, EMA_EBB_CACHE_FOLDERID, error != NULL, NULL);
 
 		talloc_free (mem_ctx);
 	}
@@ -338,31 +350,20 @@ ebbm_update_cache_cb (gpointer data)
 		if (!error && !g_cancellable_is_cancelled (priv->update_cache)) {
 			GSList *cache_keys, *c;
 
-			e_file_cache_freeze_changes (E_FILE_CACHE (priv->cache));
-			cache_keys = e_file_cache_get_keys (E_FILE_CACHE (priv->cache));
+			cache_keys = e_book_backend_sqlitedb_search_uids (priv->db, EMA_EBB_CACHE_FOLDERID, NULL, NULL, NULL);
 
 			for (c = cache_keys; c; c = c->next) {
 				const gchar *uid = c->data;
-				gchar *uid_str;
 
-				if (!uid || g_hash_table_lookup (uids, uid)
-				    || g_str_equal (uid, "populated")
-				    || g_str_equal (uid, "last_update_time")
-				    || g_str_has_prefix (uid, "key:"))
+				if (!uid || g_hash_table_lookup (uids, uid))
 					continue;
 
-				/* uid is hold by a cache, thus make a copy before remove */
-				uid_str = g_strdup (uid);
-
-				e_book_backend_mapi_notify_contact_removed (ebma, uid_str);
-
-				g_free (uid_str);
+				e_book_backend_mapi_notify_contact_removed (ebma, uid);
 			}
 
 			ebbm_set_cache_time (ebma, last_modification_secs);
-			e_file_cache_thaw_changes (E_FILE_CACHE (priv->cache));
-			e_book_backend_summary_save (priv->summary);
 
+			g_slist_foreach (cache_keys, (GFunc) g_free, NULL);
 			g_slist_free (cache_keys);
 		}
 
@@ -377,17 +378,88 @@ ebbm_update_cache_cb (gpointer data)
 	/* indicate the thread is not running */
 	g_cancellable_cancel (priv->update_cache);
 
+	/* May unref it out of the thread, in case it's the last reference to it */
+	g_idle_add (unref_backend_idle_cb, ebma);
+
 	return NULL;
 }
 
+static void
+ebbm_connect_user (EBookBackendMAPI *ebma, GCancellable *cancellable, const gchar *password, GError **error)
+{
+	EBookBackendMAPIPrivate *priv = ebma->priv;
+	GError *mapi_error = NULL;
+	ExchangeMapiConnection *old_conn;
+
+	if (!e_book_backend_is_online (E_BOOK_BACKEND (ebma))) {
+		ebbm_notify_connection_status (ebma, FALSE);
+	} else {
+		if (priv->update_cache_thread) {
+			g_cancellable_cancel (priv->update_cache);
+			g_thread_join (priv->update_cache_thread);
+			priv->update_cache_thread = NULL;
+		}
+
+		e_book_backend_mapi_lock_connection (ebma);
+
+		if (g_cancellable_set_error_if_cancelled (cancellable, error)) {
+			e_book_backend_mapi_unlock_connection (ebma);
+			return;
+		}
+
+		old_conn = priv->conn;
+		priv->conn = NULL;
+
+		priv->conn = exchange_mapi_connection_new (priv->profile,
+							   password,
+							   &mapi_error);
+		if (!priv->conn) {
+			priv->conn = exchange_mapi_connection_find (priv->profile);
+			if (priv->conn && !exchange_mapi_connection_connected (priv->conn))
+				exchange_mapi_connection_reconnect (priv->conn, password, &mapi_error);
+		}
+
+		if (old_conn)
+			g_object_unref (old_conn);
+
+		if (!priv->conn || mapi_error) {
+			if (priv->conn) {
+				g_object_unref (priv->conn);
+				priv->conn = NULL;
+			}
+
+			mapi_error_to_edb_error (error, mapi_error, E_DATA_BOOK_STATUS_OTHER_ERROR, _("Cannot connect"));
+			e_book_backend_mapi_unlock_connection (ebma);
+
+			if (mapi_error)
+				g_error_free (mapi_error);
+
+			ebbm_notify_connection_status (ebma, FALSE);
+			return;
+		}
+
+		e_book_backend_mapi_unlock_connection (ebma);
+
+		ebbm_notify_connection_status (ebma, TRUE);
+
+		if (!g_cancellable_is_cancelled (cancellable) /* && priv->marked_for_offline */) {
+			g_object_ref (ebma);
+
+			priv->update_cache_thread = g_thread_create (ebbm_update_cache_cb, ebma, TRUE, NULL);
+			if (!priv->update_cache_thread)
+				g_object_unref (ebma);
+		}
+	}
+}
+
 static void
 ebbm_open (EBookBackendMAPI *ebma, GCancellable *cancellable, gboolean only_if_exists, GError **perror)
 {
 	EBookBackendMAPIPrivate *priv = ebma->priv;
 	ESource *source = e_book_backend_get_source (E_BOOK_BACKEND (ebma));
 	const gchar *offline;
-	const gchar *cache_dir;
-	gchar *summary_file_name;
+	const gchar *cache_dir, *krb_sso;
+	GError *error = NULL;
 
 	if (e_book_backend_is_opened (E_BOOK_BACKEND (ebma))) {
 		e_book_backend_notify_opened (E_BOOK_BACKEND (ebma), NULL /* Success */);
@@ -405,24 +477,20 @@ ebbm_open (EBookBackendMAPI *ebma, GCancellable *cancellable, gboolean only_if_e
 	priv->profile = g_strdup (e_source_get_property (source, "profile"));
 
 	cache_dir = e_book_backend_get_cache_dir (E_BOOK_BACKEND (ebma));
-	summary_file_name = g_build_filename (cache_dir, "cache.summary", NULL);
-	if (priv->summary)
-		g_object_unref (priv->summary);
-	priv->summary = e_book_backend_summary_new (summary_file_name, SUMMARY_FLUSH_TIMEOUT_SECS * 1000);
 
-	if (g_file_test (summary_file_name, G_FILE_TEST_EXISTS)) {
-		if (!e_book_backend_summary_load (priv->summary))
-			g_unlink (summary_file_name);
+	if (priv->db)
+		g_object_unref (priv->db);
+	priv->db = e_book_backend_sqlitedb_new (cache_dir,
+						EMA_EBB_CACHE_PROFILEID,
+						EMA_EBB_CACHE_FOLDERID,
+						EMA_EBB_CACHE_FOLDERID,
+	                                        TRUE, &error);
+
+	if (error) {
+		g_propagate_error (perror, error);
+		return;
 	}
 
-	g_free (summary_file_name);
-
-	if (priv->cache)
-		g_object_unref (priv->cache);
-	summary_file_name = g_build_filename (cache_dir, "cache.xml", NULL);
-	priv->cache = e_book_backend_cache_new (summary_file_name);
-	g_free (summary_file_name);
-
 	e_book_backend_notify_readonly (E_BOOK_BACKEND (ebma), TRUE);
 
 	ebbm_notify_connection_status (ebma, e_book_backend_is_online (E_BOOK_BACKEND (ebma)));
@@ -443,15 +511,20 @@ ebbm_open (EBookBackendMAPI *ebma, GCancellable *cancellable, gboolean only_if_e
 	}
 
 	e_book_backend_notify_online (E_BOOK_BACKEND (ebma), TRUE);
-	e_book_backend_notify_auth_required (E_BOOK_BACKEND (ebma), TRUE, NULL);
+	krb_sso = e_source_get_property (source, "kerberos");
+	if (!krb_sso || !g_str_equal (krb_sso, "required")) {
+		e_book_backend_notify_auth_required (E_BOOK_BACKEND (ebma),
+						     TRUE, NULL);
+	} else {
+		ebbm_connect_user (ebma, cancellable, NULL, perror);
+		e_book_backend_notify_opened (E_BOOK_BACKEND (ebma), NULL);
+	}
 }
 
 static void
 ebbm_remove (EBookBackendMAPI *ebma, GCancellable *cancellable, GError **error)
 {
 	EBookBackendMAPIPrivate *priv;
-	const gchar *cache_dir;
-	gchar *filename;
 
 	e_return_data_book_error_if_fail (ebma != NULL, E_DATA_BOOK_STATUS_INVALID_ARG);
 	e_return_data_book_error_if_fail (E_IS_BOOK_BACKEND_MAPI (ebma), E_DATA_BOOK_STATUS_INVALID_ARG);
@@ -464,28 +537,23 @@ ebbm_remove (EBookBackendMAPI *ebma, GCancellable *cancellable, GError **error)
 
 	e_book_backend_mapi_lock_connection (ebma);
 
-	if (priv->summary) {
-		g_object_unref (priv->summary);
-		priv->summary = NULL;
+	if (!priv->db) {
+		const gchar *cache_dir = e_book_backend_get_cache_dir (E_BOOK_BACKEND (ebma));
+
+		/* pity, but it's required to be removed completely */
+		priv->db = e_book_backend_sqlitedb_new (cache_dir,
+							EMA_EBB_CACHE_PROFILEID,
+							EMA_EBB_CACHE_FOLDERID,
+							EMA_EBB_CACHE_FOLDERID,
+							TRUE, NULL);
 	}
 
-	if (priv->cache) {
-		g_object_unref (priv->cache);
-		priv->cache = NULL;
+	if (priv->db) {
+		e_book_backend_sqlitedb_remove (priv->db, NULL);
+		g_object_unref (priv->db);
+		priv->db = NULL;
 	}
 
-	cache_dir = e_book_backend_get_cache_dir (E_BOOK_BACKEND (ebma));
-
-	filename = g_build_filename (cache_dir, "cache.summary", NULL);
-	if (g_file_test (filename, G_FILE_TEST_EXISTS))
-			g_unlink (filename);
-	g_free (filename);
-
-	filename = g_build_filename (cache_dir, "cache.xml", NULL);
-	if (g_file_test (filename, G_FILE_TEST_EXISTS))
-			g_unlink (filename);
-	g_free (filename);
-
 	e_book_backend_mapi_unlock_connection (ebma);
 }
 
@@ -527,56 +595,13 @@ ebbm_get_backend_property (EBookBackendMAPI *ebma, const gchar *prop_name, gchar
 static void
 ebbm_authenticate_user (EBookBackendMAPI *ebma, GCancellable *cancellable, ECredentials *credentials, GError **error)
 {
-	EBookBackendMAPIPrivate *priv = ebma->priv;
-	GError *mapi_error = NULL;
-	ExchangeMapiConnection *old_conn;
+	const gchar *password;
 
 	if (!e_book_backend_is_online (E_BOOK_BACKEND (ebma))) {
 		ebbm_notify_connection_status (ebma, FALSE);
 	} else {
-		if (priv->update_cache_thread) {
-			g_cancellable_cancel (priv->update_cache);
-			g_thread_join (priv->update_cache_thread);
-			priv->update_cache_thread = NULL;
-		}
-
-		e_book_backend_mapi_lock_connection (ebma);
-
-		old_conn = priv->conn;
-		priv->conn = NULL;
-
-		priv->conn = exchange_mapi_connection_new (priv->profile, e_credentials_peek (credentials, E_CREDENTIALS_KEY_PASSWORD), &mapi_error);
-		if (!priv->conn) {
-			priv->conn = exchange_mapi_connection_find (priv->profile);
-			if (priv->conn && !exchange_mapi_connection_connected (priv->conn))
-				exchange_mapi_connection_reconnect (priv->conn, e_credentials_peek (credentials, E_CREDENTIALS_KEY_PASSWORD), &mapi_error);
-		}
-
-		if (old_conn)
-			g_object_unref (old_conn);
-
-		if (!priv->conn || mapi_error) {
-			if (priv->conn) {
-				g_object_unref (priv->conn);
-				priv->conn = NULL;
-			}
-
-			mapi_error_to_edb_error (error, mapi_error, E_DATA_BOOK_STATUS_OTHER_ERROR, _("Cannot connect"));
-			e_book_backend_mapi_unlock_connection (ebma);
-
-			if (mapi_error)
-				g_error_free (mapi_error);
-
-			ebbm_notify_connection_status (ebma, FALSE);
-			return;
-		}
-
-		e_book_backend_mapi_unlock_connection (ebma);
-
-		ebbm_notify_connection_status (ebma, TRUE);
-
-		/* if (priv->marked_for_offline) */
-			priv->update_cache_thread = g_thread_create (ebbm_update_cache_cb, ebma, TRUE, NULL);
+		password = e_credentials_peek (credentials, E_CREDENTIALS_KEY_PASSWORD);
+		ebbm_connect_user (ebma, cancellable, password, error);
 	}
 }
 
@@ -585,11 +610,14 @@ ebbm_set_online (EBookBackend *backend, gboolean is_online)
 {
 	EBookBackendMAPI *ebma = E_BOOK_BACKEND_MAPI (backend);
 	EBookBackendMAPIPrivate *priv = ebma->priv;
+	ESource *esource;
+	const gchar *krb_sso = NULL;
 
 	e_book_backend_notify_online (backend, is_online);
 	if (e_book_backend_is_opened (backend)) {
 		e_book_backend_mapi_lock_connection (ebma);
 
+		esource = e_book_backend_get_source (E_BOOK_BACKEND (ebma));
 		if (!is_online) {
 			e_book_backend_notify_readonly (backend, TRUE);
 			ebbm_notify_connection_status (ebma, FALSE);
@@ -600,8 +628,19 @@ ebbm_set_online (EBookBackend *backend, gboolean is_online)
 			}
 		} else {
 			ebbm_notify_connection_status (ebma, TRUE);
-			if (!priv->conn)
-				e_book_backend_notify_auth_required (backend, TRUE, NULL);
+			if (!priv->conn) {
+				krb_sso = e_source_get_property (esource,
+								 "kerberos");
+				if (!krb_sso
+				    || !g_str_equal (krb_sso, "required")) {
+					e_book_backend_notify_auth_required (backend, TRUE, NULL);
+				} else {
+					ebbm_connect_user (ebma, NULL, NULL,
+							   NULL);
+					e_book_backend_notify_opened (backend,
+								      NULL);
+				}
+			}
 		}
 
 		e_book_backend_mapi_unlock_connection (ebma);
@@ -612,7 +651,7 @@ static void
 ebbm_get_contact (EBookBackendMAPI *ebma, GCancellable *cancellable, const gchar *id, gchar **vcard, GError **error)
 {
 	EBookBackendMAPIPrivate *priv;
-	EContact *contact;
+	gchar *contact;
 
 	g_return_if_fail (ebma != NULL);
 	g_return_if_fail (vcard != NULL);
@@ -620,25 +659,26 @@ ebbm_get_contact (EBookBackendMAPI *ebma, GCancellable *cancellable, const gchar
 	priv = ebma->priv;
 	g_return_if_fail (priv != NULL);
 
-	if (!priv->cache) {
+	if (!priv->db) {
 		g_propagate_error (error, EDB_ERROR (REPOSITORY_OFFLINE));
 		return;
 	}
 
-	contact = e_book_backend_cache_get_contact (priv->cache, id);
-	if (contact) {
-		*vcard = e_vcard_to_string (E_VCARD (contact), EVC_FORMAT_VCARD_30);
-		g_object_unref (contact);
-	} else {
+	contact = e_book_backend_sqlitedb_get_vcard_string (priv->db,
+							    EMA_EBB_CACHE_FOLDERID,
+							    id, NULL, NULL, error);
+	if (contact)
+		*vcard = contact;
+	else
 		g_propagate_error (error, EDB_ERROR (CONTACT_NOT_FOUND));
-	}
 }
 
 static void
 ebbm_get_contact_list (EBookBackendMAPI *ebma, GCancellable *cancellable, const gchar *query, GSList **vCards, GError **error)
 {
 	EBookBackendMAPIPrivate *priv;
-	GList *contacts, *l;
+	GSList *hits, *l;
+	GError *err = NULL;
 
 	g_return_if_fail (ebma != NULL);
 	g_return_if_fail (query != NULL);
@@ -647,22 +687,28 @@ ebbm_get_contact_list (EBookBackendMAPI *ebma, GCancellable *cancellable, const
 	priv = ebma->priv;
 	g_return_if_fail (priv != NULL);
 
-	if (!priv->cache) {
+	if (!priv->db) {
 		g_propagate_error (error, EDB_ERROR (REPOSITORY_OFFLINE));
 		return;
 	}
 
-	contacts = e_book_backend_cache_get_contacts (priv->cache, query);
+	hits = e_book_backend_sqlitedb_search (priv->db, EMA_EBB_CACHE_FOLDERID,
+					       query, NULL, NULL, NULL, &err);
 
-	for (l = contacts; l; l = g_list_next (l)) {
-		EContact *contact = l->data;
+	for (l = hits; !err && l; l = l->next) {
+		EbSdbSearchData *sdata = (EbSdbSearchData *) l->data;
+		gchar *vcard = sdata->vcard;
 
-		*vCards = g_slist_prepend (*vCards, e_vcard_to_string (E_VCARD (contact), EVC_FORMAT_VCARD_30));
+		if (!err && vcard)
+			*vCards = g_slist_prepend (*vCards, g_strdup (vcard));
 
-		g_object_unref (contact);
+		e_book_backend_sqlitedb_search_data_free (sdata);
 	}
 
-	g_list_free (contacts);
+	if (err)
+		g_propagate_error (error, err);
+
+	g_slist_free (hits);
 }
 
 struct BookViewThreadData
@@ -696,14 +742,16 @@ ebbm_book_view_thread (gpointer data)
 		if (ebmac && ebmac->op_book_view_thread)
 			ebmac->op_book_view_thread (bvtd->ebma, bvtd->book_view, &error);
 
-		if (!error && !e_book_backend_cache_is_populated (priv->cache)) {
+		if (!error && !e_book_backend_sqlitedb_get_is_populated (priv->db, EMA_EBB_CACHE_FOLDERID, &error)) {
 			/* todo: create restriction based on the book_view */
-			g_cancellable_reset (priv->update_cache);
-			ebbm_fetch_contacts (bvtd->ebma, NULL, bvtd->book_view, NULL, &error);
-			g_cancellable_cancel (priv->update_cache);
+			if (!error) {
+				g_cancellable_reset (priv->update_cache);
+				ebbm_fetch_contacts (bvtd->ebma, NULL, bvtd->book_view, NULL, &error);
+				g_cancellable_cancel (priv->update_cache);
+			}
 
 			if (!error)
-				e_book_backend_cache_set_populated (priv->cache);
+				e_book_backend_sqlitedb_set_is_populated (priv->db, EMA_EBB_CACHE_FOLDERID, TRUE, &error);
 		}
 	}
 
@@ -719,7 +767,8 @@ ebbm_book_view_thread (gpointer data)
 		g_error_free (error);
 
 	g_object_unref (bvtd->book_view);
-	g_object_unref (bvtd->ebma);
+	/* May unref it out of the thread, in case it's the last reference to it */
+	g_idle_add (unref_backend_idle_cb, bvtd->ebma);
 	g_free (bvtd);
 
 	return NULL;
@@ -1004,6 +1053,7 @@ ebbm_operation_cb (OperationBase *op, gboolean cancelled, EBookBackend *backend)
 		g_object_unref (op->cancellable);
 	if (op->book)
 		g_object_unref (op->book);
+	g_object_unref (ebma);
 	g_free (op);
 }
 
@@ -1021,6 +1071,7 @@ base_op_abstract (EBookBackend *backend, EDataBook *book, guint32 opid, GCancell
 	priv = ebbm->priv;
 	g_return_if_fail (priv != NULL);
 
+	g_object_ref (ebbm);
 	if (book)
 		g_object_ref (book);
 	if (cancellable)
@@ -1049,6 +1100,7 @@ str_op_abstract (EBookBackend *backend, EDataBook *book, guint32 opid, GCancella
 	priv = ebbm->priv;
 	g_return_if_fail (priv != NULL);
 
+	g_object_ref (ebbm);
 	if (book)
 		g_object_ref (book);
 	if (cancellable)
@@ -1099,6 +1151,7 @@ ebbm_op_open (EBookBackend *backend, EDataBook *book, guint32 opid, GCancellable
 	priv = ebbm->priv;
 	g_return_if_fail (priv != NULL);
 
+	g_object_ref (ebbm);
 	if (book)
 		g_object_ref (book);
 	if (cancellable)
@@ -1130,6 +1183,7 @@ ebbm_op_remove_contacts (EBookBackend *backend, EDataBook *book, guint32 opid, G
 	priv = ebbm->priv;
 	g_return_if_fail (priv != NULL);
 
+	g_object_ref (ebbm);
 	if (book)
 		g_object_ref (book);
 	if (cancellable)
@@ -1164,6 +1218,8 @@ ebbm_op_start_book_view (EBookBackend *backend, EDataBookView *book_view)
 	priv = ebbm->priv;
 	g_return_if_fail (priv != NULL);
 
+	g_object_ref (ebbm);
+
 	op = g_new0 (OperationBookView, 1);
 	op->base.ot = OP_START_BOOK_VIEW;
 	op->base.book = NULL;
@@ -1190,6 +1246,8 @@ ebbm_op_stop_book_view (EBookBackend *backend, EDataBookView *book_view)
 	priv = ebbm->priv;
 	g_return_if_fail (priv != NULL);
 
+	g_object_ref (ebbm);
+
 	op = g_new0 (OperationBookView, 1);
 	op->base.ot = OP_STOP_BOOK_VIEW;
 	op->base.book = NULL;
@@ -1218,6 +1276,8 @@ ebbm_op_authenticate_user (EBookBackend *backend, GCancellable *cancellable, ECr
 	if (cancellable)
 		g_object_ref (cancellable);
 
+	g_object_ref (ebbm);
+
 	op = g_new0 (OperationAuthenticateUser, 1);
 	op->base.ot = OP_AUTHENTICATE_USER;
 	op->base.book = NULL;
@@ -1261,8 +1321,7 @@ ebbm_dispose (GObject *object)
 		UNREF (priv->conn);
 		e_book_backend_mapi_unlock_connection (ebma);
 		UNREF (priv->op_queue);
-		UNREF (priv->summary);
-		UNREF (priv->cache);
+		UNREF (priv->db);
 		UNREF (priv->update_cache);
 
 		FREE (priv->profile);
@@ -1372,16 +1431,13 @@ e_book_backend_mapi_get_connection (EBookBackendMAPI *ebma)
 }
 
 void
-e_book_backend_mapi_get_summary_and_cache (EBookBackendMAPI *ebma, EBookBackendSummary **summary, EBookBackendCache **cache)
+e_book_backend_mapi_get_db (EBookBackendMAPI *ebma, EBookBackendSqliteDB **db)
 {
 	g_return_if_fail (E_IS_BOOK_BACKEND_MAPI (ebma));
 	g_return_if_fail (ebma->priv != NULL);
 
-	if (summary)
-		*summary = ebma->priv->summary;
-
-	if (cache)
-		*cache = ebma->priv->cache;
+	if (db)
+		*db = ebma->priv->db;
 }
 
 gboolean
@@ -1405,10 +1461,10 @@ e_book_backend_mapi_is_marked_for_offline (EBookBackendMAPI *ebma)
 void
 e_book_backend_mapi_update_view_by_cache (EBookBackendMAPI *ebma, EDataBookView *book_view, GError **error)
 {
-	gint i;
+	gint i = 0;
 	const gchar *query = NULL;
-	EBookBackendCache *cache = NULL;
-	EBookBackendSummary *summary = NULL;
+	EBookBackendSqliteDB *db = NULL;
+	GSList *hits, *l;
 
 	g_return_if_fail (ebma != NULL);
 	g_return_if_fail (E_IS_BOOK_BACKEND_MAPI (ebma));
@@ -1416,46 +1472,30 @@ e_book_backend_mapi_update_view_by_cache (EBookBackendMAPI *ebma, EDataBookView
 	g_return_if_fail (E_IS_DATA_BOOK_VIEW (book_view));
 
 	query = e_data_book_view_get_card_query (book_view);
-	e_book_backend_mapi_get_summary_and_cache (ebma, &summary, &cache);
+	e_book_backend_mapi_get_db (ebma, &db);
 
-	if (!summary || !cache)
-		return;
+	g_return_if_fail (db != NULL);
 
-	if (e_book_backend_summary_is_summary_query (summary, query)) {
-		GPtrArray *ids = NULL;
+	hits = e_book_backend_sqlitedb_search (db, EMA_EBB_CACHE_FOLDERID,
+					       query, NULL, NULL, NULL, error);
 
-		ids = e_book_backend_summary_search (summary, query);
-		if (ids) {
-			for (i = 0; i < ids->len; i++) {
-				gchar *uid;
-				EContact *contact;
+	for (l = hits; (!error || !*error) && l; l = l->next) {
+		EbSdbSearchData *sdata = (EbSdbSearchData *) l->data;
+		gchar *vcard = sdata->vcard;
 
-				if (i > 0 && (i % 10) == 0 && !e_book_backend_mapi_book_view_is_running (ebma, book_view))
-					break;
+		if (i > 0 && ((i++) % 10) == 0 && !e_book_backend_mapi_book_view_is_running (ebma, book_view))
+			break;
 
-				uid = g_ptr_array_index (ids, i);
-				contact = e_book_backend_cache_get_contact (cache, uid);
-				if (contact) {
-					e_data_book_view_notify_update (book_view, contact);
-					g_object_unref (contact);
-				}
-			}
-
-			g_ptr_array_free (ids, TRUE);
+		if (vcard) {
+			EContact *contact = e_contact_new_from_vcard (vcard);
+			e_data_book_view_notify_update (book_view, contact);
+			g_object_unref (contact);
 		}
-	} else {
-		GList *contacts = NULL, *c;
+	}
 
-		contacts = e_book_backend_cache_get_contacts (cache, query);
-		for (c = contacts, i = 0; c != NULL; c = g_list_next (c), i++) {
-			if (i > 0 && (i % 10) == 0 && !e_book_backend_mapi_book_view_is_running (ebma, book_view))
-				break;
-
-			e_data_book_view_notify_update (book_view, E_CONTACT (c->data));
-		}
-
-		g_list_foreach (contacts, (GFunc) g_object_unref, NULL);
-		g_list_free (contacts);
+	if (hits) {
+		g_slist_foreach (hits, (GFunc) e_book_backend_sqlitedb_search_data_free, NULL);
+		g_slist_free (hits);
 	}
 }
 
@@ -1478,6 +1518,7 @@ e_book_backend_mapi_notify_contact_update (EBookBackendMAPI *ebma, EDataBookView
 	EBookBackendMAPIPrivate *priv;
 	struct FetchContactsData *fcd = notify_contact_data;
 	EDataBookView *book_view = pbook_view;
+	GError *error = NULL;
 
 	g_return_val_if_fail (E_IS_BOOK_BACKEND_MAPI (ebma), FALSE);
 	g_return_val_if_fail (ebma->priv, FALSE);
@@ -1517,22 +1558,28 @@ e_book_backend_mapi_notify_contact_update (EBookBackendMAPI *ebma, EDataBookView
 	if (!pbook_view && g_cancellable_is_cancelled (priv->update_cache))
 		return FALSE;
 
-	e_book_backend_cache_add_contact (ebma->priv->cache, contact);
-	e_book_backend_summary_add_contact (ebma->priv->summary, contact);
-	e_book_backend_notify_update (E_BOOK_BACKEND (ebma), contact);
+	e_book_backend_sqlitedb_add_contact (ebma->priv->db,
+					     EMA_EBB_CACHE_FOLDERID, contact,
+					     FALSE, &error);
+	if (!error) {
+		e_book_backend_notify_update (E_BOOK_BACKEND (ebma), contact);
 
-	if (fcd && pr_last_modification_time) {
-		if (fcd->last_modification < pr_last_modification_time->tv_sec)
-			fcd->last_modification = pr_last_modification_time->tv_sec;
+		if (fcd && pr_last_modification_time) {
+			if (fcd->last_modification < pr_last_modification_time->tv_sec)
+				fcd->last_modification = pr_last_modification_time->tv_sec;
+		}
+		return TRUE;
 	}
-
-	return TRUE;
+	g_error_free (error);
+	return FALSE;
 }
 
 void
 e_book_backend_mapi_notify_contact_removed (EBookBackendMAPI *ebma, const gchar *uid)
 {
 	EBookBackendMAPIPrivate *priv;
+	GError *error = NULL;
+	gboolean ret;
 
 	g_return_if_fail (E_IS_BOOK_BACKEND_MAPI (ebma));
 	g_return_if_fail (ebma->priv);
@@ -1541,57 +1588,38 @@ e_book_backend_mapi_notify_contact_removed (EBookBackendMAPI *ebma, const gchar
 	priv = ebma->priv;
 	g_return_if_fail (priv != NULL);
 
-	e_book_backend_cache_remove_contact (priv->cache, uid);
-	e_book_backend_summary_remove_contact (priv->summary, uid);
-	e_book_backend_notify_remove (E_BOOK_BACKEND (ebma), uid);
-}
+	ret = e_book_backend_sqlitedb_remove_contact (priv->db,
+						      EMA_EBB_CACHE_FOLDERID,
+						      uid, &error);
+	if (ret && !error)
+		e_book_backend_notify_remove (E_BOOK_BACKEND (ebma), uid);
 
-static gchar *
-create_cache_key (const gchar *key)
-{
-	g_return_val_if_fail (key != NULL, NULL);
-
-	return g_strconcat ("key:", key, NULL);
+	if (error)
+		g_error_free (error);
 }
 
 void
 e_book_backend_mapi_cache_set (EBookBackendMAPI *ebma, const gchar *key, const gchar *value)
 {
-	gchar *real_key;
-
 	g_return_if_fail (ebma != NULL);
 	g_return_if_fail (E_IS_BOOK_BACKEND_MAPI (ebma));
 	g_return_if_fail (ebma->priv != NULL);
-	g_return_if_fail (ebma->priv->cache != NULL);
+	g_return_if_fail (ebma->priv->db != NULL);
 	g_return_if_fail (key != NULL);
 
-	real_key = create_cache_key (key);
-	g_return_if_fail (real_key != NULL);
-
-	if (!e_file_cache_add_object (E_FILE_CACHE (ebma->priv->cache), real_key, value))
-		e_file_cache_replace_object (E_FILE_CACHE (ebma->priv->cache), real_key, value);
-
-	g_free (real_key);
+	e_book_backend_sqlitedb_set_key_value (ebma->priv->db, EMA_EBB_CACHE_FOLDERID, key, value, NULL);
 }
 
 gchar *
 e_book_backend_mapi_cache_get (EBookBackendMAPI *ebma, const gchar *key)
 {
-	gchar *real_key, *res;
-
 	g_return_val_if_fail (ebma != NULL, NULL);
 	g_return_val_if_fail (E_IS_BOOK_BACKEND_MAPI (ebma), NULL);
 	g_return_val_if_fail (ebma->priv != NULL, NULL);
-	g_return_val_if_fail (ebma->priv->cache != NULL, NULL);
+	g_return_val_if_fail (ebma->priv->db != NULL, NULL);
 	g_return_val_if_fail (key != NULL, NULL);
 
-	real_key = create_cache_key (key);
-	g_return_val_if_fail (real_key != NULL, NULL);
-
-	res = g_strdup (e_file_cache_get_object (E_FILE_CACHE (ebma->priv->cache), real_key));
-	g_free (real_key);
-
-	return res;
+	return e_book_backend_sqlitedb_get_key_value (ebma->priv->db, EMA_EBB_CACHE_FOLDERID, key, NULL);
 }
 
 /* utility functions/macros */
@@ -1818,23 +1846,35 @@ mapi_book_utils_contact_from_props (ExchangeMapiConnection *conn, mapi_id_t fid,
 			value = (gpointer) get_proptag (mappings[i].mapi_id);
 		contact_type = mappings[i].contact_type & ELEMENT_TYPE_MASK;
 		if (mappings[i].element_type == PT_UNICODE && contact_type == ELEMENT_TYPE_SIMPLE) {
-			if (value)
-				e_contact_set (contact, mappings[i].field_id, value);
+			const gchar *str = value;
+			if (str && *str)
+				e_contact_set (contact, mappings[i].field_id, str);
 		} else if (contact_type == ELEMENT_TYPE_SIMPLE) {
 			if (value && mappings[i].element_type == PT_SYSTIME) {
 				const struct FILETIME *t = value;
-				gchar buff[129];
-				time_t time = exchange_mapi_util_filetime_to_time_t (t);
+				gchar *buff = NULL;
+				GTimeVal tv;
 
-				e_contact_set (contact, mappings[i].field_id, ctime_r (&time, buff));
+				tv.tv_sec = exchange_mapi_util_filetime_to_time_t (t);
+				tv.tv_usec = 0;
+				
+				buff = g_time_val_to_iso8601 (&tv);
+
+				if (buff)
+					e_contact_set (contact, mappings[i].field_id, buff);
+
+				g_free (buff);
 			}
 		} else if (contact_type == ELEMENT_TYPE_COMPLEX) {
 			if (mappings[i].field_id == E_CONTACT_IM_AIM) {
-				GList *list = g_list_append (NULL, value);
+				const gchar *str = value;
+				if (str && *str) {
+					GList *list = g_list_append (NULL, (gpointer) str);
 
-				e_contact_set (contact, mappings[i].field_id, list);
+					e_contact_set (contact, mappings[i].field_id, list);
 
-				g_list_free (list);
+					g_list_free (list);
+				}
 			} else if (mappings[i].field_id == E_CONTACT_BIRTH_DATE
 				   || mappings[i].field_id == E_CONTACT_ANNIVERSARY) {
 				const struct FILETIME *t = value;
@@ -1860,7 +1900,7 @@ mapi_book_utils_contact_from_props (ExchangeMapiConnection *conn, mapi_id_t fid,
 				if (mappings[i].field_id == E_CONTACT_ADDRESS_HOME) {
 					contact_addr.address_format = NULL;
 					contact_addr.po = NULL;
-					contact_addr.street = (gchar *)value;
+					contact_addr.street = (gchar *) value;
 					contact_addr.ext = (gchar *) get_str_proptag (PR_HOME_ADDRESS_POST_OFFICE_BOX_UNICODE);
 					contact_addr.locality = (gchar *) get_str_proptag (PR_HOME_ADDRESS_CITY_UNICODE);
 					contact_addr.region = (gchar *) get_str_proptag (PR_HOME_ADDRESS_STATE_OR_PROVINCE_UNICODE);
@@ -1869,14 +1909,26 @@ mapi_book_utils_contact_from_props (ExchangeMapiConnection *conn, mapi_id_t fid,
 				} else {
 					contact_addr.address_format = NULL;
 					contact_addr.po = NULL;
-					contact_addr.street = (gchar *)value;
+					contact_addr.street = (gchar *) value;
 					contact_addr.ext = (gchar *) get_str_proptag (PR_POST_OFFICE_BOX_UNICODE);
 					contact_addr.locality = (gchar *) get_str_proptag (PR_LOCALITY_UNICODE);
 					contact_addr.region = (gchar *) get_str_proptag (PR_STATE_OR_PROVINCE_UNICODE);
 					contact_addr.code = (gchar *) get_str_proptag (PR_POSTAL_CODE_UNICODE);
 					contact_addr.country = (gchar *) get_str_proptag (PR_COUNTRY_UNICODE);
 				}
-				e_contact_set (contact, mappings[i].field_id, &contact_addr);
+
+				#define is_set(x) ((x) && *(x))
+				if (is_set (contact_addr.address_format) ||
+				    is_set (contact_addr.po) ||
+				    is_set (contact_addr.street) ||
+				    is_set (contact_addr.ext) ||
+				    is_set (contact_addr.locality) ||
+				    is_set (contact_addr.region) ||
+				    is_set (contact_addr.code) ||
+				    is_set (contact_addr.country)) {
+					e_contact_set (contact, mappings[i].field_id, &contact_addr);
+				}
+				#undef is_set
 			}
 		}
 	}
