@@ -26,6 +26,7 @@
 #endif
 
 #include <glib/gstdio.h>
+#include <glib/gi18n-lib.h>
 #include <fcntl.h>
 #include <libecal/e-cal-util.h>
 #include "exchange-mapi-cal-utils.h"
@@ -270,7 +271,7 @@ exchange_mapi_cal_util_fetch_attachments (ECalComponent *comp, GSList **attach_l
 			g_mapped_file_free (mapped_file);
 #endif
 		} else if (error) {
-			g_debug ("Could not map %s: %s \n", sfname_uri, error->message);
+			exchange_mapi_debug_print ("Could not map %s: %s \n", sfname_uri, error->message);
 			g_error_free (error);
 		}
 
@@ -476,10 +477,10 @@ set_attachments_to_cal_component (ECalComponent *comp, GSList *attach_list, cons
 		fd = g_open (filename, O_RDWR|O_CREAT|O_TRUNC|O_BINARY, 0600);
 		if (fd == -1) {
 			/* skip gracefully */
-			g_debug ("Could not open %s for writing \n", filename);
+			exchange_mapi_debug_print ("Could not open %s for writing \n", filename);
 		} else if (len && write (fd, attach, len) == -1) {
 			/* skip gracefully */
-			g_debug ("Attachment write failed \n");
+			exchange_mapi_debug_print ("Attachment write failed \n");
 		}
 		if (fd != -1) {
 			close (fd);
@@ -669,7 +670,7 @@ exchange_mapi_cal_util_mapi_props_to_comp (ExchangeMapiConnection *conn, mapi_id
 	struct timeval t;
 	ExchangeMAPIStream *body_stream;
 	const gchar *subject = NULL, *body = NULL;
-	const struct StringArray_r *categories_array;
+	const struct StringArrayW_r *categories_array;
 	const uint32_t *ui32;
 	const bool *b;
 	icalcomponent *ical_comp;
@@ -730,7 +731,7 @@ exchange_mapi_cal_util_mapi_props_to_comp (ExchangeMapiConnection *conn, mapi_id
 		gint ii;
 
 		for (ii = 0; ii < categories_array->cValues; ii++) {
-			const gchar *category = categories_array->lppszA[ii];
+			const gchar *category = categories_array->lppszW[ii];
 
 			if (!category || !*category)
 				continue;
@@ -750,7 +751,7 @@ exchange_mapi_cal_util_mapi_props_to_comp (ExchangeMapiConnection *conn, mapi_id
 		ExchangeMAPIStream *stream;
 
 		/* GlobalObjectId */
-		stream = exchange_mapi_util_find_stream (streams, PidLidGlobalObjectId);
+		stream = exchange_mapi_util_find_stream_namedid (streams, conn, fid, PidLidGlobalObjectId);
 		if (stream) {
 			gchar *value = globalid_to_string (stream->value);
 			prop = icalproperty_new_x (value);
@@ -780,7 +781,7 @@ exchange_mapi_cal_util_mapi_props_to_comp (ExchangeMapiConnection *conn, mapi_id
 		}
 
 		/* AppointmentSequence */
-		ui32 = (const uint32_t *)find_mapi_SPropValue_data(properties, PidLidAppointmentSequence);
+		ui32 = exchange_mapi_util_find_array_namedid (properties, conn, fid, PidLidAppointmentSequence);
 		if (ui32) {
 			gchar *value = g_strdup_printf ("%d", *ui32);
 			prop = icalproperty_new_x (value);
@@ -789,21 +790,21 @@ exchange_mapi_cal_util_mapi_props_to_comp (ExchangeMapiConnection *conn, mapi_id
 			g_free (value);
 		}
 
-		location = (const gchar *)exchange_mapi_util_find_array_propval(properties, PidLidLocation);
+		location = exchange_mapi_util_find_array_namedid (properties, conn, fid, PidLidLocation);
 		if (location && *location)
 			icalcomponent_set_location (ical_comp, location);
 
-		b = (const bool *)find_mapi_SPropValue_data(properties, PidLidAppointmentSubType);
+		b = exchange_mapi_util_find_array_namedid (properties, conn, fid, PidLidAppointmentSubType);;
 		all_day = b && *b;
 
-		stream = exchange_mapi_util_find_stream (streams, PidLidAppointmentTimeZoneDefinitionStartDisplay);
+		stream = exchange_mapi_util_find_stream_namedid (streams, conn, fid, PidLidAppointmentTimeZoneDefinitionStartDisplay);
 		if (stream) {
 			gchar *buf = exchange_mapi_cal_util_bin_to_mapi_tz (stream->value);
 			dtstart_tz_location = exchange_mapi_cal_tz_util_get_ical_equivalent (buf);
 			g_free (buf);
 		}
 
-		if (get_mapi_SPropValue_array_date_timeval (&t, properties, PidLidAppointmentStartWhole) == MAPI_E_SUCCESS) {
+		if (exchange_mapi_util_find_array_datetime_namedid (&t, properties, conn, fid, PidLidAppointmentStartWhole) == MAPI_E_SUCCESS) {
 			icaltimezone *zone = dtstart_tz_location ? icaltimezone_get_builtin_timezone (dtstart_tz_location) : (icaltimezone *)default_zone;
 			prop = icalproperty_new_dtstart (icaltime_from_timet_with_zone (t.tv_sec, all_day, zone));
 			if (!all_day && zone && icaltimezone_get_tzid (zone)) {
@@ -813,14 +814,14 @@ exchange_mapi_cal_util_mapi_props_to_comp (ExchangeMapiConnection *conn, mapi_id
 			icalcomponent_add_property (ical_comp, prop);
 		}
 
-		stream = exchange_mapi_util_find_stream (streams, PidLidAppointmentTimeZoneDefinitionEndDisplay);
+		stream = exchange_mapi_util_find_stream_namedid (streams, conn, fid, PidLidAppointmentTimeZoneDefinitionEndDisplay);
 		if (stream) {
 			gchar *buf = exchange_mapi_cal_util_bin_to_mapi_tz (stream->value);
 			dtend_tz_location = exchange_mapi_cal_tz_util_get_ical_equivalent (buf);
 			g_free (buf);
 		}
 
-		if (get_mapi_SPropValue_array_date_timeval (&t, properties, PidLidAppointmentEndWhole) == MAPI_E_SUCCESS) {
+		if (exchange_mapi_util_find_array_datetime_namedid (&t, properties, conn, fid, PidLidAppointmentEndWhole) == MAPI_E_SUCCESS) {
 			icaltimezone *zone;
 
 			if (!dtend_tz_location)
@@ -835,7 +836,7 @@ exchange_mapi_cal_util_mapi_props_to_comp (ExchangeMapiConnection *conn, mapi_id
 			icalcomponent_add_property (ical_comp, prop);
 		}
 
-		ui32 = (const uint32_t *)find_mapi_SPropValue_data(properties, PidLidBusyStatus);
+		ui32 = exchange_mapi_util_find_array_namedid (properties, conn, fid, PidLidBusyStatus);
 		if (ui32) {
 			prop = icalproperty_new_transp (get_transp_from_prop (*ui32));
 			icalcomponent_add_property (ical_comp, prop);
@@ -903,7 +904,7 @@ exchange_mapi_cal_util_mapi_props_to_comp (ExchangeMapiConnection *conn, mapi_id
 					param = icalparameter_new_cn (name);
 					icalproperty_add_parameter (prop, param);
 
-					ui32 = exchange_mapi_util_find_array_propval (properties, PidLidResponseStatus);
+					ui32 = exchange_mapi_util_find_array_namedid (properties, conn, fid, PidLidResponseStatus);
 					param = icalparameter_new_partstat (get_partstat_from_trackstatus (ui32 ? *ui32 : olResponseNone));
 					icalproperty_add_parameter (prop, param);
 
@@ -949,9 +950,9 @@ exchange_mapi_cal_util_mapi_props_to_comp (ExchangeMapiConnection *conn, mapi_id
 			}
 		}
 
-		b = (const bool *)find_mapi_SPropValue_data(properties, PidLidRecurring);
+		b = exchange_mapi_util_find_array_namedid (properties, conn, fid, PidLidRecurring);
 		if (b && *b) {
-			stream = exchange_mapi_util_find_stream (streams, PidLidAppointmentRecur);
+			stream = exchange_mapi_util_find_stream_namedid (streams, conn, fid, PidLidAppointmentRecur);
 			if (stream) {
 				icaltimezone *recur_zone;
 				const gchar *recur_tz_location;
@@ -965,12 +966,12 @@ exchange_mapi_cal_util_mapi_props_to_comp (ExchangeMapiConnection *conn, mapi_id
 			}
 		}
 
-		b = (const bool *)find_mapi_SPropValue_data(properties, PidLidReminderSet);
+		b = exchange_mapi_util_find_array_namedid (properties, conn, fid, PidLidReminderSet);
 		if (b && *b) {
 			struct timeval start, displaytime;
 
-			if ((get_mapi_SPropValue_array_date_timeval (&start, properties, PidLidReminderTime) == MAPI_E_SUCCESS)
-			 && (get_mapi_SPropValue_array_date_timeval (&displaytime, properties, PidLidReminderSignalTime) == MAPI_E_SUCCESS)) {
+			if ((exchange_mapi_util_find_array_datetime_namedid (&start, properties, conn, fid, PidLidReminderTime) == MAPI_E_SUCCESS)
+			 && (exchange_mapi_util_find_array_datetime_namedid (&displaytime, properties, conn, fid, PidLidReminderSignalTime) == MAPI_E_SUCCESS)) {
 				ECalComponentAlarm *e_alarm = e_cal_component_alarm_new ();
 				ECalComponentAlarmTrigger trigger;
 
@@ -991,38 +992,38 @@ exchange_mapi_cal_util_mapi_props_to_comp (ExchangeMapiConnection *conn, mapi_id
 		const uint64_t *status = NULL;
 
 		/* NOTE: Exchange tasks are DATE values, not DATE-TIME values, but maybe someday, we could expect Exchange to support it;) */
-		if (get_mapi_SPropValue_array_date_timeval (&t, properties, PidLidTaskStartDate) == MAPI_E_SUCCESS)
+		if (exchange_mapi_util_find_array_datetime_namedid (&t, properties, conn, fid, PidLidTaskStartDate) == MAPI_E_SUCCESS)
 			icalcomponent_set_dtstart (ical_comp, icaltime_from_timet_with_zone (t.tv_sec, 1, default_zone));
-		if (get_mapi_SPropValue_array_date_timeval (&t, properties, PidLidTaskDueDate) == MAPI_E_SUCCESS)
+		if (exchange_mapi_util_find_array_datetime_namedid (&t, properties, conn, fid, PidLidTaskDueDate) == MAPI_E_SUCCESS)
 			icalcomponent_set_due (ical_comp, icaltime_from_timet_with_zone (t.tv_sec, 1, default_zone));
 
-		status = (const uint64_t *)find_mapi_SPropValue_data(properties, PidLidTaskStatus);
+		status = exchange_mapi_util_find_array_namedid (properties, conn, fid, PidLidTaskStatus);
 		if (status) {
 			icalcomponent_set_status (ical_comp, get_taskstatus_from_prop(*status));
 			if (*status == olTaskComplete
-			&& get_mapi_SPropValue_array_date_timeval (&t, properties, PidLidTaskDateCompleted) == MAPI_E_SUCCESS) {
+			&& exchange_mapi_util_find_array_datetime_namedid (&t, properties, conn, fid, PidLidTaskDateCompleted) == MAPI_E_SUCCESS) {
 				prop = icalproperty_new_completed (icaltime_from_timet_with_zone (t.tv_sec, 1, default_zone));
 				icalcomponent_add_property (ical_comp, prop);
 			}
 		}
 
-		complete = (const double *)find_mapi_SPropValue_data(properties, PidLidPercentComplete);
+		complete = exchange_mapi_util_find_array_namedid (properties, conn, fid, PidLidPercentComplete);
 		if (complete) {
 			prop = icalproperty_new_percentcomplete ((gint)(*complete * 100 + 1e-9));
 			icalcomponent_add_property (ical_comp, prop);
 		}
 
-		b = (const bool *)find_mapi_SPropValue_data(properties, PidLidTaskFRecurring);
+		b = exchange_mapi_util_find_array_namedid (properties, conn, fid, PidLidTaskFRecurring);
 		if (b && *b) {
 			/* FIXME: Evolution does not support recurring tasks */
 			g_warning ("Encountered a recurring task.");
 		}
 
-		b = (const bool *)find_mapi_SPropValue_data(properties, PidLidReminderSet);
+		b = exchange_mapi_util_find_array_namedid (properties, conn, fid, PidLidReminderSet);
 		if (b && *b) {
 			struct timeval abs;
 
-			if (get_mapi_SPropValue_array_date_timeval (&abs, properties, PidLidReminderTime) == MAPI_E_SUCCESS) {
+			if (exchange_mapi_util_find_array_datetime_namedid (&abs, properties, conn, fid, PidLidReminderTime) == MAPI_E_SUCCESS) {
 				ECalComponentAlarm *e_alarm = e_cal_component_alarm_new ();
 				ECalComponentAlarmTrigger trigger;
 
@@ -1240,7 +1241,8 @@ appt_build_name_id (ExchangeMapiConnection *conn, mapi_id_t fid, TALLOC_CTX *mem
 		{ PidLidIsException, 0 },
 		{ PidLidCleanGlobalObjectId, 0 },
 		{ PidLidAppointmentMessageClass, 0 },
-		{ PidLidMeetingType, 0 }
+		{ PidLidMeetingType, 0 },
+		{ PidLidTimeZone, 0 }
 	};
 
 	if (!props)
@@ -1267,9 +1269,9 @@ task_build_name_id (ExchangeMapiConnection *conn, mapi_id_t fid, TALLOC_CTX *mem
 		{ PidLidTaskOwner, 0 },
 		{ PidLidTaskAssigner, 0 },
 		{ PidLidTaskFRecurring, 0 },
-		{ PidLidTaskRole, 0 },
 		{ PidLidTaskOwnership, 0 },
-		{ PidLidTaskAcceptanceState, 0 }
+		{ PidLidTaskAcceptanceState, 0 },
+		{ PidLidTaskRole, 0 }
 	};
 
 	if (!props)
@@ -1372,7 +1374,7 @@ exchange_mapi_cal_utils_write_props_cb (ExchangeMapiConnection *conn, mapi_id_t
 		} G_STMT_END
 
 	#define set_named_value(named_id, val) G_STMT_START { \
-		if (!exchange_mapi_utils_add_spropvalue_named_id (conn, fid, mem_ctx, values, n_values, named_id, val)) \
+		if (!exchange_mapi_utils_add_spropvalue_namedid (conn, fid, mem_ctx, values, n_values, named_id, val)) \
 			return FALSE;	\
 		} G_STMT_END
 
@@ -1390,7 +1392,7 @@ exchange_mapi_cal_utils_write_props_cb (ExchangeMapiConnection *conn, mapi_id_t
 									\
 		exchange_mapi_util_time_t_to_filetime (dtval, &filetime); \
 									\
-		if (!exchange_mapi_utils_add_spropvalue_named_id (conn, fid, mem_ctx, values, n_values, named_id, &filetime)) \
+		if (!exchange_mapi_utils_add_spropvalue_namedid (conn, fid, mem_ctx, values, n_values, named_id, &filetime)) \
 			return FALSE;	\
 		} G_STMT_END
 
@@ -1469,11 +1471,11 @@ exchange_mapi_cal_utils_write_props_cb (ExchangeMapiConnection *conn, mapi_id_t
 	if (categories) {
 		gint ii;
 		GSList *c;
-		struct StringArray_r *categories_array;
+		struct StringArrayW_r *categories_array;
 
-		categories_array = talloc_zero (mem_ctx, struct StringArray_r);
+		categories_array = talloc_zero (mem_ctx, struct StringArrayW_r);
 		categories_array->cValues = g_slist_length (categories);
-		categories_array->lppszA = (const char **) talloc_zero_array (mem_ctx, gchar *, categories_array->cValues);
+		categories_array->lppszW = (const char **) talloc_zero_array (mem_ctx, gchar *, categories_array->cValues);
 
 		for (c = categories, ii = 0; c; c = c->next, ii++) {
 			const gchar *category = c->data;
@@ -1484,7 +1486,7 @@ exchange_mapi_cal_utils_write_props_cb (ExchangeMapiConnection *conn, mapi_id_t
 				continue;
 			}
 
-			categories_array->lppszA[ii] = talloc_strdup (mem_ctx, category);
+			categories_array->lppszW[ii] = talloc_strdup (mem_ctx, category);
 		}
 
 		set_named_value (PidNameKeywords, categories_array);
@@ -1652,6 +1654,21 @@ exchange_mapi_cal_utils_write_props_cb (ExchangeMapiConnection *conn, mapi_id_t
 			set_named_value (PidLidAppointmentTimeZoneDefinitionEndDisplay, &end_tz);
 		}
 
+		/* Recurrences also need to have this rather arbitrary index set
+		   to properly determine SDT/DST and appear in OWA (Bug #629057). */
+		if (e_cal_component_has_recurrences (comp)) {
+			uint64_t pltz;
+			icaltimezone *ictz;
+			const gchar *zone_location = dtstart_tz_location;
+
+			if (!zone_location)
+				zone_location = get_tzid_location ("*default-zone*", cbdata);
+
+			ictz = icaltimezone_get_builtin_timezone (zone_location);
+			pltz = exchange_mapi_cal_util_mapi_tz_pidlidtimezone (ictz);
+			set_named_value (PidLidTimeZone, &pltz);
+		}
+
 		/* Duration */
 		flag32 = icaldurationtype_as_int (icaltime_subtract (dtend, dtstart));
 		flag32 /= MINUTES_IN_HOUR;
@@ -1807,15 +1824,36 @@ exchange_mapi_cal_utils_write_props_cb (ExchangeMapiConnection *conn, mapi_id_t
 
 			break;
 		case MEETING_RESPONSE :
+			#define prefix_subject(prefix) {					\
+				const gchar *summary;						\
+												\
+				summary = icalcomponent_get_summary (ical_comp);		\
+				if (!(summary && *summary))					\
+					summary = "";						\
+												\
+				summary = talloc_asprintf (mem_ctx, "%s %s", prefix, summary);	\
+												\
+				set_value (PR_SUBJECT_UNICODE, summary);			\
+				set_value (PR_NORMALIZED_SUBJECT_UNICODE, summary);		\
+				if (cbdata->appt_seq == 0)					\
+					set_value (PR_CONVERSATION_TOPIC_UNICODE, summary);	\
+			}
 			if (cbdata->resp == olResponseAccepted) {
+				/* Translators: This is a meeting response prefix which will be shown in a message Subject */
+				prefix_subject (C_("MeetingResp", "Accepted:"));
 				text = IPM_SCHEDULE_MEETING_RESP_POS;
 			} else if (cbdata->resp == olResponseTentative) {
+				/* Translators: This is a meeting response prefix which will be shown in a message Subject */
+				prefix_subject (C_("MeetingResp", "Tentative:"));
 				text = IPM_SCHEDULE_MEETING_RESP_TENT;
 			} else if (cbdata->resp == olResponseDeclined) {
+				/* Translators: This is a meeting response prefix which will be shown in a message Subject */
+				prefix_subject (C_("MeetingResp", "Declined:"));
 				text = IPM_SCHEDULE_MEETING_RESP_NEG;
 			} else {
 				text = "";
 			}
+			#undef prefix_subject
 			set_value (PR_MESSAGE_CLASS, text);
 			text = NULL;
 
@@ -1966,11 +2004,9 @@ exchange_mapi_cal_util_get_new_appt_id (ExchangeMapiConnection *conn, mapi_id_t
 			TALLOC_CTX *mem_ctx = talloc_init ("ExchangeMAPI_get_new_appt_id");
 
 			set_SPropValue_proptag (&sprop, PR_OWNER_APPT_ID, (gconstpointer ) &id);
-			cast_mapi_SPropValue (
-				#ifdef HAVE_MEMCTX_ON_CAST_MAPI_SPROPVALUE
-				mem_ctx,
-				#endif
-				&(res.res.resProperty.lpProp), &sprop);
+			cast_mapi_SPropValue (mem_ctx,
+					      &(res.res.resProperty.lpProp),
+					      &sprop);
 			ids = exchange_mapi_connection_check_restriction (conn, fid, 0, &res, NULL);
 			if (ids) {
 				GSList *l;
@@ -2027,7 +2063,7 @@ mapi_get_date_from_string (const gchar *dtstring)
 }
 
 static void
-populate_freebusy_data (struct Binary_r *bin, uint32_t month, uint32_t year, GList **freebusy, const gchar *accept_type, ECalComponent *comp)
+populate_freebusy_data (struct Binary_r *bin, uint32_t month, uint32_t year, GSList **freebusy, const gchar *accept_type, ECalComponent *comp)
 {
 	uint16_t	event_start;
 	uint16_t	event_end;
@@ -2113,13 +2149,13 @@ populate_freebusy_data (struct Binary_r *bin, uint32_t month, uint32_t year, GLi
 }
 
 gboolean
-exchange_mapi_cal_utils_get_free_busy_data (ExchangeMapiConnection *conn, const GList *users, time_t start, time_t end, GList **freebusy, GError **mapi_error)
+exchange_mapi_cal_utils_get_free_busy_data (ExchangeMapiConnection *conn, const GSList *users, time_t start, time_t end, GSList **freebusy, GError **mapi_error)
 {
 	struct SRow		aRow;
 	enum MAPISTATUS		ms;
 	uint32_t		i;
 	mapi_object_t           obj_store;
-	const GList *l;
+	const GSList *l;
 
 	const uint32_t			*publish_start;
 	const struct LongArray_r	*busy_months;
@@ -2138,11 +2174,13 @@ exchange_mapi_cal_utils_get_free_busy_data (ExchangeMapiConnection *conn, const
 	icaltimetype start_time, end_time;
 	icaltimezone *default_zone = NULL;
 
+	*freebusy = NULL;
+
 	if (!exchange_mapi_connection_get_public_folder (conn, &obj_store, mapi_error)) {
 		return FALSE;
 	}
 
-	for ( l = users; l != NULL; l = g_list_next (l)) {
+	for ( l = users; l != NULL; l = g_slist_next (l)) {
 		ms = GetUserFreeBusyData (&obj_store, (const gchar *)l->data, &aRow);
 
 		if (ms != MAPI_E_SUCCESS) {
@@ -2214,7 +2252,7 @@ exchange_mapi_cal_utils_get_free_busy_data (ExchangeMapiConnection *conn, const
 		}
 
 		e_cal_component_commit_sequence (comp);
-		*freebusy = g_list_append (*freebusy, e_cal_component_get_as_string (comp));
+		*freebusy = g_slist_append (*freebusy, e_cal_component_get_as_string (comp));
 		g_object_unref (comp);
 		MAPIFreeBuffer(aRow.lpProps);
 	}
